@@ -1,11 +1,19 @@
-# Copyright (c) 2016-2018 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2016-2018 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -444,6 +452,9 @@ class SSHAgentClient:
     def lock(self, passphrase):
         """Lock the agent using the specified passphrase
 
+           .. note:: The lock and unlock actions don't appear to be
+                     supported on the Windows 10 OpenSSH agent.
+
            :param passphrase:
                The passphrase required to later unlock the agent
            :type passphrase: `str`
@@ -466,6 +477,9 @@ class SSHAgentClient:
     def unlock(self, passphrase):
         """Unlock the agent using the specified passphrase
 
+           .. note:: The lock and unlock actions don't appear to be
+                     supported on the Windows 10 OpenSSH agent.
+
            :param passphrase:
                The passphrase to use to unlock the agent
            :type passphrase: `str`
