@@ -1,11 +1,19 @@
-# Copyright (c) 2013-2018 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2013-2018 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -606,6 +614,14 @@ class SSHChannel(SSHPacketHandler):
             raise DisconnectError(DISC_PROTOCOL_ERROR,
                                   'Unexpected channel response')
 
+    def _process_keepalive_at_openssh_dot_com_request(self, packet):
+        """Process an incoming OpenSSH keepalive request"""
+
+        packet.check_end()
+
+        self.logger.debug2('Received OpenSSH keepalive channel request')
+        return True
+
     _packet_handlers = {
         MSG_CHANNEL_WINDOW_ADJUST:      _process_window_adjust,
         MSG_CHANNEL_DATA:               _process_data,
@@ -736,11 +752,27 @@ class SSHChannel(SSHPacketHandler):
            See :meth:`get_extra_info() <SSHClientConnection.get_extra_info>`
            on :class:`SSHClientConnection` for more information.
 
+           Additional information stored on the channel by calling
+           :meth:`set_extra_info` can also be returned here.
+
         """
 
         return self._extra.get(name, self._conn.get_extra_info(name, default)
                                if self._conn else default)
 
+    def set_extra_info(self, **kwargs):
+        """Store additional information associated with the channel
+
+           This method allows extra information to be associated with the
+           channel. The information to store should be passed in as
+           keyword parameters and can later be returned by calling
+           :meth:`get_extra_info` with one of the keywords as the name
+           to retrieve.
+
+        """
+
+        self._extra.update(**kwargs)
+
     def can_write_eof(self):
         """Return whether the channel supports :meth:`write_eof`
 
@@ -1034,7 +1066,7 @@ class SSHClientChannel(SSHChannel):
                     raise ValueError('Invalid pty mode: %s' % mode)
 
                 name = _pty_mode_names.get(mode, str(mode))
-                self.logger.debug1('  Mode %s: %s', name, value)
+                self.logger.debug2('  Mode %s: %s', name, value)
                 modes += Byte(mode) + UInt32(value)
 
             modes += Byte(PTY_OP_END)
@@ -1234,12 +1266,12 @@ class SSHClientChannel(SSHChannel):
            process or service. Signal names should be as described in
            section 6.10 of :rfc:`RFC 4254 <4254#section-6.10>`.
 
-           .. note:: OpenSSH's SSH server implementation does not
-                     currently support this message, so attempts to
+           .. note:: OpenSSH's SSH server implementation prior to version
+                     7.9 does not support this message, so attempts to
                      use :meth:`send_signal`, :meth:`terminate`, or
-                     :meth:`kill` with an OpenSSH SSH server will
-                     end up being ignored. This is currently being
-                     tracked in OpenSSH `bug 1424`__.
+                     :meth:`kill` with an older OpenSSH SSH server will
+                     end up being ignored. This was tracked in OpenSSH
+                     `bug 1424`__.
 
                      __ https://bugzilla.mindrot.org/show_bug.cgi?id=1424
 
@@ -1368,7 +1400,7 @@ class SSHServerChannel(SSHChannel):
             if idx+4 <= len(modes):
                 name = _pty_mode_names.get(mode, str(mode))
                 value = int.from_bytes(modes[idx:idx+4], 'big')
-                self.logger.debug1('  Mode %s: %s', name, value)
+                self.logger.debug2('  Mode %s: %s', name, value)
                 term_modes[mode] = value
                 idx += 4
             else:
@@ -1819,9 +1851,9 @@ class SSHTCPChannel(SSHForwardChannel):
                   orig_host, orig_port):
         """Open a TCP channel"""
 
-        self._extra['peername'] = (None, None)
-        self._extra['local_peername'] = (orig_host, orig_port)
-        self._extra['remote_peername'] = (host, port)
+        self.set_extra_info(peername=(None, None),
+                            local_peername=(orig_host, orig_port),
+                            remote_peername=(host, port))
 
         return (yield from super()._open_forward(session_factory, chantype,
                                                  String(host), UInt32(port),
@@ -1846,8 +1878,8 @@ class SSHTCPChannel(SSHForwardChannel):
                                orig_host, orig_port):
         """Set local and remote peer names for inbound connections"""
 
-        self._extra['local_peername'] = (dest_host, dest_port)
-        self._extra['remote_peername'] = (orig_host, orig_port)
+        self.set_extra_info(local_peername=(dest_host, dest_port),
+                            remote_peername=(orig_host, orig_port))
 
 
 class SSHUNIXChannel(SSHForwardChannel):
@@ -1857,8 +1889,7 @@ class SSHUNIXChannel(SSHForwardChannel):
     def _open_unix(self, session_factory, chantype, path, *args):
         """Open a UNIX channel"""
 
-        self._extra['local_peername'] = ''
-        self._extra['remote_peername'] = path
+        self.set_extra_info(local_peername='', remote_peername=path)
 
         return (yield from super()._open_forward(session_factory, chantype,
                                                  String(path), *args))
@@ -1885,8 +1916,7 @@ class SSHUNIXChannel(SSHForwardChannel):
     def set_inbound_peer_names(self, dest_path):
         """Set local and remote peer names for inbound connections"""
 
-        self._extra['local_peername'] = dest_path
-        self._extra['remote_peername'] = ''
+        self.set_extra_info(local_peername=dest_path, remote_peername='')
 
 
 class SSHX11Channel(SSHForwardChannel):
@@ -1896,8 +1926,8 @@ class SSHX11Channel(SSHForwardChannel):
     def open(self, session_factory, orig_host, orig_port):
         """Open an SSH X11 channel"""
 
-        self._extra['local_peername'] = (orig_host, orig_port)
-        self._extra['remote_peername'] = (None, None)
+        self.set_extra_info(local_peername=(orig_host, orig_port),
+                            remote_peername=(None, None))
 
         return (yield from self._open_forward(session_factory, b'x11',
                                               String(orig_host),
@@ -1906,8 +1936,8 @@ class SSHX11Channel(SSHForwardChannel):
     def set_inbound_peer_names(self, orig_host, orig_port):
         """Set local and remote peer name for inbound connections"""
 
-        self._extra['local_peername'] = (None, None)
-        self._extra['remote_peername'] = (orig_host, orig_port)
+        self.set_extra_info(local_peername=(None, None),
+                            remote_peername=(orig_host, orig_port))
 
 
 class SSHAgentChannel(SSHForwardChannel):
