@@ -1,11 +1,19 @@
-# Copyright (c) 2013-2015 by Ron Frederick <ronf@timeheart.net>.
-# All rights reserved.
+# Copyright (c) 2013-2018 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
-# the terms of the Eclipse Public License v1.0 which accompanies this
+# the terms of the Eclipse Public License v2.0 which accompanies this
 # distribution and is available at:
 #
-#     http://www.eclipse.org/legal/epl-v10.html
+#     http://www.eclipse.org/legal/epl-2.0/
+#
+# This program may also be made available under the following secondary
+# licenses when the conditions for such availability set forth in the
+# Eclipse Public License v2.0 are satisfied:
+#
+#    GNU General Public License, Version 2.0, or any later versions of
+#    that license
+#
+# SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 #
 # Contributors:
 #     Ron Frederick - initial implementation, API, and documentation
@@ -16,19 +24,6 @@
 class SSHSession:
     """SSH session handler"""
 
-    def connection_made(self, chan):
-        """Called when a channel is opened successfully
-
-           This method is called when a channel is opened successfully. The
-           channel parameter should be stored if needed for later use.
-
-           :param chan:
-               The channel which was successfully opened.
-           :type chan: :class:`SSHClientChannel`
-
-        """
-        pass # pragma: no cover
-
     def connection_lost(self, exc):
         """Called when a channel is closed
 
@@ -141,6 +136,20 @@ class SSHClientSession(SSHSession):
 
     """
 
+    def connection_made(self, chan):
+        """Called when a channel is opened successfully
+
+           This method is called when a channel is opened successfully. The
+           channel parameter should be stored if needed for later use.
+
+           :param chan:
+               The channel which was successfully opened.
+           :type chan: :class:`SSHClientChannel`
+
+        """
+
+        pass # pragma: no cover
+
     def xon_xoff_requested(self, client_can_do):
         """XON/XOFF flow control has been enabled or disabled
 
@@ -232,6 +241,20 @@ class SSHServerSession(SSHSession):
 
     """
 
+    def connection_made(self, chan):
+        """Called when a channel is opened successfully
+
+           This method is called when a channel is opened successfully. The
+           channel parameter should be stored if needed for later use.
+
+           :param chan:
+               The channel which was successfully opened.
+           :type chan: :class:`SSHServerChannel`
+
+        """
+
+        pass # pragma: no cover
+
     def pty_requested(self, term_type, term_size, term_modes):
         """A psuedo-terminal has been requested
 
@@ -451,6 +474,20 @@ class SSHTCPSession(SSHSession):
 
     """
 
+    def connection_made(self, chan):
+        """Called when a channel is opened successfully
+
+           This method is called when a channel is opened successfully. The
+           channel parameter should be stored if needed for later use.
+
+           :param chan:
+               The channel which was successfully opened.
+           :type chan: :class:`SSHTCPChannel`
+
+        """
+
+        pass # pragma: no cover
+
 
 class SSHUNIXSession(SSHSession):
     """SSH UNIX domain socket session handler
@@ -483,3 +520,17 @@ class SSHUNIXSession(SSHSession):
        Received data will be passed to the :meth:`data_received` method.
 
     """
+
+    def connection_made(self, chan):
+        """Called when a channel is opened successfully
+
+           This method is called when a channel is opened successfully. The
+           channel parameter should be stored if needed for later use.
+
+           :param chan:
+               The channel which was successfully opened.
+           :type chan: :class:`SSHUNIXChannel`
+
+        """
+
+        pass # pragma: no cover
